@@ -2,7 +2,7 @@
 //
 /****************************************************************************
    liblscp - LinuxSampler Control Protocol API
-   Copyright (C) 2004-2006, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -44,6 +44,7 @@ typedef struct _lscp_server_info_t
 {
 	char *        description;
 	char *        version;
+	char *        protocol_version;
 
 } lscp_server_info_t;
 
@@ -63,7 +64,7 @@ typedef struct _lscp_channel_info_t
 	char *        engine_name;
 	int           audio_device;
 	int           audio_channels;
-	char **       audio_routing;
+	int  *        audio_routing;
 	char *        instrument_file;
 	int           instrument_nr;
 	char *        instrument_name;
@@ -71,6 +72,7 @@ typedef struct _lscp_channel_info_t
 	int           midi_device;
 	int           midi_port;
 	int           midi_channel;
+	int           midi_map;
 	float         volume;
 	int           mute;
 	int           solo;
@@ -96,12 +98,23 @@ typedef enum _lscp_usage_t
 } lscp_usage_t;
 
 
+/** Effect send info cache struct. */
+typedef struct _lscp_fxsend_info_t
+{
+	char *        name;
+	int           midi_controller;
+	int *         audio_routing;
+	float         level;
+
+} lscp_fxsend_info_t;
+
+
 /** MIDI instrument parameter struct. */
 typedef struct _lscp_midi_instrument_t
 {
-	int bank_msb;
-	int bank_lsb;
-	int program;
+	int           map;
+	int           bank;
+	int           prog;
 
 } lscp_midi_instrument_t;
 
@@ -131,6 +144,16 @@ typedef struct _lscp_midi_instrument_info_t
 } lscp_midi_instrument_info_t;
 
 
+/** MIDI instrument map mode. */
+typedef enum _lscp_midi_map_mode_t
+{
+	LSCP_MIDI_MAP_NONE = -1,
+	LSCP_MIDI_MAP_DEFAULT = -2,
+	LSCP_MIDI_MAP_ALL = -3
+
+} lscp_midi_map_mode_t;
+
+
 //-------------------------------------------------------------------------
 // Client socket main structure.
 
@@ -148,7 +171,7 @@ typedef lscp_status_t (*lscp_client_proc_t)
 );
 
 //-------------------------------------------------------------------------
-// Client versioning teller fuunction.
+// Client versioning teller function.
 
 const char *            lscp_client_package             (void);
 const char *            lscp_client_version             (void);
@@ -210,6 +233,8 @@ lscp_status_t           lscp_set_channel_midi_type      (lscp_client_t *pClient,
 lscp_status_t           lscp_set_channel_midi_device    (lscp_client_t *pClient, int iSamplerChannel, int iMidiDevice);
 lscp_status_t           lscp_set_channel_midi_port      (lscp_client_t *pClient, int iSamplerChannel, int iMidiPort);
 lscp_status_t           lscp_set_channel_midi_channel   (lscp_client_t *pClient, int iSamplerChannel, int iMidiChannel);
+lscp_status_t           lscp_set_channel_midi_map       (lscp_client_t *pClient, int iSamplerChannel, int iMidiMap);
+
 lscp_status_t           lscp_set_channel_volume         (lscp_client_t *pClient, int iSamplerChannel, float fVolume);
 
 lscp_status_t           lscp_set_channel_mute           (lscp_client_t *pClient, int iSamplerChannel, int iMute);
@@ -224,19 +249,50 @@ lscp_server_info_t *    lscp_get_server_info            (lscp_client_t *pClient)
 int                     lscp_get_total_voice_count      (lscp_client_t *pClient);
 int                     lscp_get_total_voice_count_max  (lscp_client_t *pClient);
 
+float                   lscp_get_volume                 (lscp_client_t *pClient );
+lscp_status_t           lscp_set_volume                 (lscp_client_t *pClient, float fVolume);
+
+//-------------------------------------------------------------------------
+// Effect sends control functions.
+
+int                     lscp_create_fxsend              (lscp_client_t *pClient, int iSamplerChannel, int iMidiController, const char *pszFxName);
+lscp_status_t           lscp_destroy_fxsend             (lscp_client_t *pClient, int iSamplerChannel, int iFxSend);
+
+int                     lscp_get_fxsends                (lscp_client_t *pClient, int iSamplerChannel);
+int *                   lscp_list_fxsends               (lscp_client_t *pClient, int iSamplerChannel);
+
+lscp_fxsend_info_t *    lscp_get_fxsend_info            (lscp_client_t *pClient, int iSamplerChannel, int iFxSend);
+
+lscp_status_t           lscp_set_fxsend_audio_channel   (lscp_client_t *pClient, int iSamplerChannel, int iFxSend, int iAudioSrc, int iAudioDst);
+lscp_status_t           lscp_set_fxsend_midi_controller (lscp_client_t *pClient, int iSamplerChannel, int iFxSend, int iMidiController);
+lscp_status_t           lscp_set_fxsend_level           (lscp_client_t *pClient, int iSamplerChannel, int iFxSend, float fLevel);
+
 //-------------------------------------------------------------------------
 // MIDI instrument mapping control functions.
 
+int                     lscp_add_midi_instrument_map    (lscp_client_t *pClient, const char *pszMapName);
+lscp_status_t           lscp_remove_midi_instrument_map (lscp_client_t *pClient, int iMidiMap);
+
+int                     lscp_get_midi_instrument_maps   (lscp_client_t *pClient);
+int *                   lscp_list_midi_instrument_maps  (lscp_client_t *pClient);
+
+const char *            lscp_get_midi_instrument_map_name (lscp_client_t *pClient, int iMidiMap);
+lscp_status_t           lscp_set_midi_instrument_map_name (lscp_client_t *pClient, int iMidiMap, const char *pszMapName);
+
 lscp_status_t           lscp_map_midi_instrument        (lscp_client_t *pClient, lscp_midi_instrument_t *pMidiInstr, const char *pszEngineName, const char *pszFileName, int iInstrIndex, float fVolume, lscp_load_mode_t load_mode, const char *pszName);
 lscp_status_t           lscp_unmap_midi_instrument      (lscp_client_t *pClient, lscp_midi_instrument_t *pMidiInstr);
 
-int                     lscp_get_midi_instruments       (lscp_client_t *pClient);
-lscp_midi_instrument_t *lscp_list_midi_instruments      (lscp_client_t *pClient);
+int                     lscp_get_midi_instruments       (lscp_client_t *pClient, int iMidiMap);
+lscp_midi_instrument_t *lscp_list_midi_instruments      (lscp_client_t *pClient, int iMidiMap);
 
 lscp_midi_instrument_info_t *lscp_get_midi_instrument_info(lscp_client_t *pClient, lscp_midi_instrument_t *pMidiInstr);
 
-lscp_status_t           lscp_clear_midi_instruments     (lscp_client_t *pClient);
+lscp_status_t           lscp_clear_midi_instruments     (lscp_client_t *pClient, int iMidiMap);
 
+//-------------------------------------------------------------------------
+// Instrument editor functions.
+
+lscp_status_t           lscp_edit_channel_instrument    (lscp_client_t *pClient, int iSamplerChannel);
 
 #if defined(__cplusplus)
 }
