@@ -2,7 +2,7 @@
 //
 /****************************************************************************
    liblscp - LinuxSampler Control Protocol API
-   Copyright (C) 2004-2006, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -33,14 +33,22 @@ extern "C" {
 /** Subscribable event notification bit-wise flags. */
 typedef enum _lscp_event_t
 {
-	LSCP_EVENT_NONE              = 0x0000,
-	LSCP_EVENT_CHANNEL_COUNT     = 0x0001,
-	LSCP_EVENT_VOICE_COUNT       = 0x0002,
-	LSCP_EVENT_STREAM_COUNT      = 0x0004,
-	LSCP_EVENT_BUFFER_FILL       = 0x0008,
-	LSCP_EVENT_CHANNEL_INFO      = 0x0010,
-	LSCP_EVENT_TOTAL_VOICE_COUNT = 0x0020,
-	LSCP_EVENT_MISCELLANEOUS     = 0x1000
+	LSCP_EVENT_NONE                      = 0x0000,
+	LSCP_EVENT_CHANNEL_COUNT             = 0x0001,
+	LSCP_EVENT_VOICE_COUNT               = 0x0002,
+	LSCP_EVENT_STREAM_COUNT              = 0x0004,
+	LSCP_EVENT_BUFFER_FILL               = 0x0008,
+	LSCP_EVENT_CHANNEL_INFO              = 0x0010,
+	LSCP_EVENT_TOTAL_VOICE_COUNT         = 0x0020,
+	LSCP_EVENT_AUDIO_OUTPUT_DEVICE_COUNT = 0x0040,
+	LSCP_EVENT_AUDIO_OUTPUT_DEVICE_INFO  = 0x0080,
+	LSCP_EVENT_MIDI_INPUT_DEVICE_COUNT   = 0x0100,
+	LSCP_EVENT_MIDI_INPUT_DEVICE_INFO    = 0x0200,
+	LSCP_EVENT_MIDI_INSTRUMENT_MAP_COUNT = 0x0400,
+	LSCP_EVENT_MIDI_INSTRUMENT_MAP_INFO  = 0x1000,
+	LSCP_EVENT_MIDI_INSTRUMENT_COUNT     = 0x2000,
+	LSCP_EVENT_MIDI_INSTRUMENT_INFO      = 0x4000,
+	LSCP_EVENT_MISCELLANEOUS             = 0x8000
 
 } lscp_event_t;
 
