@@ -2,7 +2,7 @@
 //
 /****************************************************************************
    liblscp - LinuxSampler Control Protocol API
-   Copyright (C) 2004-2006, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2004-2007, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -275,6 +275,9 @@ int lscp_get_available_audio_drivers ( lscp_client_t *pClient )
 {
 	int iAudioDrivers = -1;
 
+	if (pClient == NULL)
+		return -1;
+
 	// Lock this section up.
 	lscp_mutex_lock(pClient->mutex);
 
@@ -301,6 +304,9 @@ const char ** lscp_list_available_audio_drivers ( lscp_client_t *pClient )
 {
 	const char *pszSeps = ",";
 
+	if (pClient == NULL)
+		return NULL;
+
 	// Lock this section up.
 	lscp_mutex_lock(pClient->mutex);
 
@@ -389,9 +395,9 @@ int lscp_create_audio_device ( lscp_client_t *pClient, const char *pszAudioDrive
 	int iAudioDevice = -1;
 
 	if (pClient == NULL)
-		return iAudioDevice;
+		return -1;
 	if (pszAudioDriver == NULL)
-		return iAudioDevice;
+		return -1;
 
 	// Lock this section up.
 	lscp_mutex_lock(pClient->mutex);
@@ -445,6 +451,9 @@ int lscp_get_audio_devices ( lscp_client_t *pClient )
 {
 	int iAudioDevices = -1;
 
+	if (pClient == NULL)
+		return -1;
+
 	// Lock this section up.
 	lscp_mutex_lock(pClient->mutex);
 
@@ -644,6 +653,9 @@ int lscp_get_available_midi_drivers ( lscp_client_t *pClient )
 {
 	int iMidiDrivers = -1;
 
+	if (pClient == NULL)
+		return -1;
+
 	// Lock this section up.
 	lscp_mutex_lock(pClient->mutex);
 
@@ -670,6 +682,9 @@ const char** lscp_list_available_midi_drivers ( lscp_client_t *pClient )
 {
 	const char *pszSeps = ",";
 
+	if (pClient == NULL)
+		return NULL;
+
 	// Lock this section up.
 	lscp_mutex_lock(pClient->mutex);
 
@@ -760,9 +775,9 @@ int lscp_create_midi_device ( lscp_client_t *pClient, const char *pszMidiDriver,
 	int iMidiDevice = -1;
 
 	if (pClient == NULL)
-		return iMidiDevice;
+		return -1;
 	if (pszMidiDriver == NULL)
-		return iMidiDevice;
+		return -1;
 
 	// Lock this section up.
 	lscp_mutex_lock(pClient->mutex);
@@ -816,6 +831,9 @@ int lscp_get_midi_devices ( lscp_client_t *pClient )
 {
 	int iMidiDevices = -1;
 
+	if (pClient == NULL)
+		return -1;
+
 	// Lock this section up.
 	lscp_mutex_lock(pClient->mutex);
 
